@@ -29,7 +29,7 @@ public:
     DebuggerMainWnd(const char* name);
     ~DebuggerMainWnd();
 
-    bool debugProgram(const QString& exe, QCString lang);
+    bool debugProgram(const QString& exe, const QString& lang);
 
 protected:
     // session properties
@@ -39,9 +39,6 @@ protected:
     void saveSettings(KConfig*);
     void restoreSettings(KConfig*);
 
-    // statusbar texts
-    void updateLineStatus(int lineNo);	/* zero-based line number */
-
     void initToolbar();
     void initKAction();
 
@@ -62,7 +59,7 @@ protected:
     KRecentFilesAction* m_recentExecAction;
 
 protected:
-    virtual void closeEvent(QCloseEvent* e);
+    virtual bool queryClose();
     virtual TTYWindow* ttyWindow();
     virtual QString createOutputWindow();
 
@@ -85,8 +82,6 @@ signals:
 public slots:
     virtual void updateUI();
     virtual void updateLineItems();
-    void slotFileChanged();
-    void slotLineChanged();
     void slotAddWatch();
     void slotAddWatch(const QString& text);
     void slotNewFileLoaded();
@@ -103,12 +98,10 @@ public slots:
     void slotEditValue();
 
     void slotFileOpen();
-    void slotFileQuit();
     void slotFileExe();
     void slotFileCore();
     void slotFileGlobalSettings();
     void slotFileProgSettings();
-    void slotViewToolbar();
     void slotViewStatusbar();
     void slotExecUntil();
     void slotExecAttach();
