@@ -4,7 +4,7 @@
  * See the file COPYING in the toplevel directory of the source directory.
  */
 
-#include <kapp.h>
+#include <kapplication.h>
 #include <klocale.h>			/* i18n */
 #include <kconfig.h>
 #include <kmessagebox.h>
@@ -21,7 +21,6 @@
 #include "prefmisc.h"
 #include "ttywnd.h"
 #include "commandids.h"
-#include "valarray.h"
 #ifdef HAVE_CONFIG
 #include "config.h"
 #endif
@@ -114,19 +113,6 @@ void WatchWindow::slotWatchHighlighted()
 }
 
 
-static void splitCmdStr(const QString& cmd, ValArray<QString>& parts)
-{
-    QString str = cmd.simplifyWhiteSpace();
-    int start = 0;
-    int end;
-    while ((end = str.find(' ', start)) >= 0) {
-	parts.append(str.mid(start, end-start));
-	start = end+1;
-    }
-    parts.append(str.mid(start, str.length()-start));
-}
-
-
 const char defaultTermCmdStr[] = "xterm -name kdbgio -title %T -e sh -c %C";
 const char defaultSourceFilter[] = "*.c *.cc *.cpp *.c++ *.C *.CC";
 const char defaultHeaderFilter[] = "*.h *.hh *.hpp *.h++";
@@ -194,7 +180,13 @@ void DebuggerMainWndBase::setRemoteDevice(const QString& remoteDevice)
     }
 }
 
-void DebuggerMainWndBase::setTranscript(const char* name)
+void DebuggerMainWndBase::overrideProgramArguments(const QString& args)
+{
+    assert(m_debugger != 0);
+    m_debugger->overrideProgramArguments(args);
+}
+
+void DebuggerMainWndBase::setTranscript(const QString& name)
 {
     m_transcriptFile = name;
     if (m_debugger != 0 && m_debugger->driver() != 0)
@@ -268,11 +260,11 @@ void DebuggerMainWndBase::setAttachPid(const QString& pid)
 }
 
 bool DebuggerMainWndBase::debugProgram(const QString& executable,
-				       QCString lang, QWidget* parent)
+				       QString lang, QWidget* parent)
 {
     assert(m_debugger != 0);
 
-    TRACE(QString().sprintf("trying language '%s'...", lang.data()));
+    TRACE(QString("trying language '%1'...").arg(lang));
     DebuggerDriver* driver = driverFromLang(lang);
 
     if (driver == 0)
@@ -288,20 +280,19 @@ bool DebuggerMainWndBase::debugProgram(const QString& executable,
 	    // The config file exists but doesn't have an entry,
 	    // so it must have been created by an old version of KDbg
 	    // that had only the GDB driver.
-	    lang = c.readEntry(KDebugger::DriverNameEntry, "GDB").latin1();
+	    lang = c.readEntry(KDebugger::DriverNameEntry, "GDB");
 
-	    TRACE(QString().sprintf("...bad, trying config driver %s...",
-			  lang.data()));
+	    TRACE(QString("...bad, trying config driver %1...").arg(lang));
 	    driver = driverFromLang(lang);
 	}
 
     }
     if (driver == 0)
     {
-	QCString name = driverNameFromFile(executable);
+	QString name = driverNameFromFile(executable);
 
-	TRACE(QString().sprintf("...no luck, trying %s derived"
-				" from file contents", name.data()));
+	TRACE(QString("...no luck, trying %1 derived"
+		" from file contents").arg(name));
 	driver = driverFromLang(name);
     }
     if (driver == 0)
@@ -329,7 +320,7 @@ bool DebuggerMainWndBase::debugProgram(const QString& executable,
 }
 
 // derive driver from language
-DebuggerDriver* DebuggerMainWndBase::driverFromLang(QCString lang)
+DebuggerDriver* DebuggerMainWndBase::driverFromLang(QString lang)
 {
     // lang is needed in all lowercase
     lang = lang.lower();
@@ -357,12 +348,11 @@ DebuggerDriver* DebuggerMainWndBase::driverFromLang(QCString lang)
 	const L& l = langs[i];
 
 	// shortest must match
-	if (strncmp(l.shortest, lang, strlen(l.shortest)) != 0)
+	if (!lang.startsWith(l.shortest))
 	    continue;
 
 	// lang must not be longer than the full name, and it must match
-	if (lang.length() <= strlen(l.full) &&
-	    strncmp(l.full, lang, lang.length()) == 0)
+	if (QString(l.full).startsWith(lang))
 	{
 	    driverID = l.driver;
 	    break;
@@ -390,7 +380,7 @@ DebuggerDriver* DebuggerMainWndBase::driverFromLang(QCString lang)
 /**
  * Try to guess the language to use from the contents of the file.
  */
-QCString DebuggerMainWndBase::driverNameFromFile(const QString& exe)
+QString DebuggerMainWndBase::driverNameFromFile(const QString& exe)
 {
     /* Inprecise but simple test to see if file is in XSLT language */
     if (exe.right(4).lower() == ".xsl")
@@ -474,15 +464,15 @@ QString DebuggerMainWndBase::createOutputWindow()
     fifoName.sprintf(fifoNameBase, ::getpid());
 
     // create a fifo that will pass in the tty name
-    ::unlink(fifoName);			/* remove remnants */
+    QFile::remove(fifoName);		// remove remnants
 #ifdef HAVE_MKFIFO
-    if (::mkfifo(fifoName, S_IRUSR|S_IWUSR) < 0) {
+    if (::mkfifo(fifoName.local8Bit(), S_IRUSR|S_IWUSR) < 0) {
 	// failed
 	TRACE("mkfifo " + fifoName + " failed");
 	return QString();
     }
 #else
-    if (::mknod(fifoName, S_IFIFO | S_IRUSR|S_IWUSR, 0) < 0) {
+    if (::mknod(fifoName.local8Bit(), S_IFIFO | S_IRUSR|S_IWUSR, 0) < 0) {
 	// failed
 	TRACE("mknod " + fifoName + " failed");
 	return QString();
@@ -502,21 +492,21 @@ QString DebuggerMainWndBase::createOutputWindow()
 	    "exec<&-;exec>&-;"		/* close stdin and stdout */
 	    "while :;do sleep 3600;done";
 	// let config file override this script
-	const char* fmt = shellScriptFmt;
-	if (m_outputTermKeepScript.length() != 0) {
-	    fmt = m_outputTermKeepScript.data();
+	QString shellScript;
+	if (!m_outputTermKeepScript.isEmpty()) {
+	    shellScript = m_outputTermKeepScript;
+	} else {
+	    shellScript = shellScriptFmt;
 	}
 
-	QString shellScript;
-	shellScript.sprintf(fmt, fifoName.data());
+	shellScript.replace("%s", fifoName);
 	TRACE("output window script is " + shellScript);
 
 	QString title = kapp->caption();
 	title += i18n(": Program output");
 
 	// parse the command line specified in the preferences
-	ValArray<QString> cmdParts;
-	splitCmdStr(m_outputTermCmdStr, cmdParts);
+	QStringList cmdParts = QStringList::split(' ', m_outputTermCmdStr);
 
 	/*
 	 * Build the argv array. Thereby substitute special sequences:
@@ -529,8 +519,9 @@ QString DebuggerMainWndBase::createOutputWindow()
 	    { "%C", shellScript }
 	};
 
-	for (int i = 0; i < cmdParts.size(); i++) {
-	    QString& str = cmdParts[i];
+	for (QStringList::iterator i = cmdParts.begin(); i != cmdParts.end(); ++i)
+	{
+	    QString& str = *i;
 	    for (int j = sizeof(substitute)/sizeof(substitute[0])-1; j >= 0; j--) {
 		int pos = str.find(substitute[j].seq);
 		if (pos >= 0) {
@@ -545,28 +536,20 @@ QString DebuggerMainWndBase::createOutputWindow()
 
     if (m_outputTermProc->start())
     {
+	QString tty;
+
 	// read the ttyname from the fifo
-	int f = ::open(fifoName, O_RDONLY);
-	if (f < 0) {
-	    // error
-	    ::unlink(fifoName);
-	    return QString();
-	}
-
-	char ttyname[50];
-	int n = ::read(f, ttyname, sizeof(ttyname)-sizeof(char));   /* leave space for '\0' */
-
-	::close(f);
-	::unlink(fifoName);
-
-	if (n < 0) {
-	    // error
-	    return QString();
+	QFile f(fifoName);
+	if (f.open(IO_ReadOnly))
+	{
+	    QByteArray t = f.readAll();
+	    tty = QString::fromLocal8Bit(t, t.size());
+	    f.close();
 	}
+	f.remove();
 
 	// remove whitespace
-	ttyname[n] = '\0';
-	QString tty = QString(ttyname).stripWhiteSpace();
+	tty = tty.stripWhiteSpace();
 	TRACE("tty=" + tty);
 	return tty;
     }
@@ -574,7 +557,7 @@ QString DebuggerMainWndBase::createOutputWindow()
     {
 	// error, could not start xterm
 	TRACE("fork failed for fifo " + fifoName);
-	::unlink(fifoName);
+	QFile::remove(fifoName);
 	shutdownTermWindow();
 	return QString();
     }
