@@ -7,15 +7,14 @@
 #ifndef WINSTACK_H
 #define WINSTACK_H
 
-#include <qptrlist.h>
 #include <qdialog.h>
 #include <qlineedit.h>
 #include <qlayout.h>
 #include <qcheckbox.h>
 #include <qpushbutton.h>
-#include <qpopupmenu.h>
 #include <qtooltip.h>
-#include "valarray.h"
+#include <ktabwidget.h>
+#include <list>
 
 // forward declarations
 class KDebugger;
@@ -61,22 +60,13 @@ public:
 };
 
 
-class WinStack : public QWidget
+class WinStack : public KTabWidget
 {
     Q_OBJECT
 public:
     WinStack(QWidget* parent, const char* name);
     virtual ~WinStack();
 
-    enum { WindowMore=0x100, WindowMask=0xf };
-
-    /**
-     * The menu set with setWindowMenu will be modified by this widget to
-     * list the available windows. The specified popup menu must be set up
-     * to contain an entry with ID WindowMore. The windows will be inserted
-     * before this entry.
-     */
-    void setWindowMenu(QPopupMenu* menu);
     /**
      * Slot activate also looks in this directory when the specified file is
      * a relative path.
@@ -86,15 +76,14 @@ public:
     bool activeLine(QString& filename, int& lineNo);
     bool activeLine(QString& filename, int& lineNo, DbgAddr& address);
     void maybeTip(const QPoint& p);
-    bool hasWindows() const { return m_fileList.size() > 0; }
+    bool hasWindows() const { return count() > 0; }
     QString activeFileName() const;
+    SourceWindow* activeWindow() const;
+    SourceWindow* windowAt(int i) const;
 
     virtual QSize sizeHint() const;
-    virtual void resizeEvent(QResizeEvent*);
 
 signals:
-    void fileChanged();
-    void lineChanged();
     void toggleBreak(const QString&, int, const DbgAddr&, bool);
     void enadisBreak(const QString&, int, const DbgAddr&);
     void newFileLoaded();
@@ -104,7 +93,6 @@ signals:
     void moveProgramCounter(const QString&, int, const DbgAddr&);
 
 public slots:
-    void selectWindow(int id);		/* 1-based index, 0 means dialog More... */
     virtual void slotFindForward();
     virtual void slotFindBackward();
     virtual void activate(const QString& filename, int lineNo, const DbgAddr& address);
@@ -119,13 +107,14 @@ public slots:
     void slotBrkptSetTemp();
     void slotBrkptEnable();
     void slotMoveProgramCounter();
+    void slotClose();
 
     // Displays the value tip at m_tipLocation
     void slotShowValueTip(const QString& tipText);
 
     // Shows the disassembled code at the location given by file and lineNo
     void slotDisassembled(const QString& fileName, int lineNo,
-			  const QList<DisassembledCode>& disass);
+			  const std::list<DisassembledCode>& disass);
 
     // Updates line items after expanding/collapsing disassembled code
     void slotExpandCollapse(int lineNo);
@@ -133,14 +122,11 @@ public slots:
 protected:
     bool activatePath(QString pathname, int lineNo, const DbgAddr& address);
     virtual bool activateWindow(SourceWindow* fw, int lineNo, const DbgAddr& address);	/* -1 doesnt change line */
-    virtual void changeWindowMenu();
     virtual void contextMenuEvent(QContextMenuEvent* e);
     void setPC(bool set, const QString& fileName, int lineNo,
 	       const DbgAddr& address, int frameNo);
-    ValArray<SourceWindow*> m_fileList;
-    SourceWindow* m_activeWindow;
+    SourceWindow* findByFileName(const QString& fileName) const;
     QString m_lastOpenDir;		/* where user opened last file */
-    QPopupMenu* m_windowMenu;
     
     // program counter
     QString m_pcFile;
